import { EditorState, Transaction, TransactionSpec, Extension, SelectionRange, StateEffect, Facet, Line, EditorStateConfig } from "@codemirror/state";
import { StyleModule, StyleSpec } from "style-mod";
import { DocView } from "./docview";
import { InputState } from "./input";
import { Rect, ScrollStrategy } from "./dom";
import { BlockInfo } from "./heightmap";
import { ViewState } from "./viewstate";
import { ViewUpdate, AttrSource, ViewPlugin, PluginValue, MeasureRequest } from "./extension";
import { DOMObserver } from "./domobserver";
import { BidiSpan, Direction } from "./bidi";
export interface EditorViewConfig extends EditorStateConfig {
    state?: EditorState;
    parent?: Element | DocumentFragment;
    root?: Document | ShadowRoot;
    dispatchTransactions?: (trs: readonly Transaction[], view: EditorView) => void;
    dispatch?: (tr: Transaction, view: EditorView) => void;
}
export declare const enum UpdateState {
    Idle = 0,
    Measuring = 1,
    Updating = 2
}
export declare class EditorView {
    get state(): EditorState;
    get viewport(): {
        from: number;
        to: number;
    };
    get visibleRanges(): readonly {
        from: number;
        to: number;
    }[];
    get inView(): boolean;
    get composing(): boolean;
    get compositionStarted(): boolean;
    private dispatchTransactions;
    private _root;
    get root(): DocumentOrShadowRoot;
    get win(): Window & typeof globalThis;
    readonly dom: HTMLElement;
    readonly scrollDOM: HTMLElement;
    readonly contentDOM: HTMLElement;
    private announceDOM;
    inputState: InputState;
    viewState: ViewState;
    docView: DocView;
    private plugins;
    private pluginMap;
    private editorAttrs;
    private contentAttrs;
    private styleModules;
    private bidiCache;
    private destroyed;
    updateState: UpdateState;
    observer: DOMObserver;
    measureScheduled: number;
    measureRequests: MeasureRequest<any>[];
    constructor(config?: EditorViewConfig);
    dispatch(tr: Transaction): void;
    dispatch(trs: readonly Transaction[]): void;
    dispatch(...specs: TransactionSpec[]): void;
    update(transactions: readonly Transaction[]): void;
    setState(newState: EditorState): void;
    private updatePlugins;
    measure(flush?: boolean): void;
    get themeClasses(): string;
    private updateAttrs;
    private showAnnouncements;
    private mountStyles;
    private readMeasured;
    requestMeasure<T>(request?: MeasureRequest<T>): void;
    plugin<T extends PluginValue>(plugin: ViewPlugin<T>): T | null;
    get documentTop(): number;
    get documentPadding(): {
        top: number;
        bottom: number;
    };
    get scaleX(): number;
    get scaleY(): number;
    elementAtHeight(height: number): BlockInfo;
    lineBlockAtHeight(height: number): BlockInfo;
    get viewportLineBlocks(): BlockInfo[];
    lineBlockAt(pos: number): BlockInfo;
    get contentHeight(): number;
    moveByChar(start: SelectionRange, forward: boolean, by?: (initial: string) => (next: string) => boolean): SelectionRange;
    moveByGroup(start: SelectionRange, forward: boolean): SelectionRange;
    moveToLineBoundary(start: SelectionRange, forward: boolean, includeWrap?: boolean): SelectionRange;
    moveVertically(start: SelectionRange, forward: boolean, distance?: number): SelectionRange;
    domAtPos(pos: number): {
        node: Node;
        offset: number;
    };
    posAtDOM(node: Node, offset?: number): number;
    posAtCoords(coords: {
        x: number;
        y: number;
    }, precise: false): number;
    posAtCoords(coords: {
        x: number;
        y: number;
    }): number | null;
    coordsAtPos(pos: number, side?: -1 | 1): Rect | null;
    coordsForChar(pos: number): Rect | null;
    get defaultCharacterWidth(): number;
    get defaultLineHeight(): number;
    get textDirection(): Direction;
    textDirectionAt(pos: number): Direction;
    get lineWrapping(): boolean;
    bidiSpans(line: Line): readonly BidiSpan[];
    get hasFocus(): boolean;
    focus(): void;
    setRoot(root: Document | ShadowRoot): void;
    destroy(): void;
    static scrollIntoView(pos: number | SelectionRange, options?: {
        y?: ScrollStrategy;
        x?: ScrollStrategy;
        yMargin?: number;
        xMargin?: number;
    }): StateEffect<unknown>;
    static styleModule: Facet<StyleModule, readonly StyleModule[]>;
    static domEventHandlers(handlers: DOMEventHandlers<any>): Extension;
    static inputHandler: Facet<(view: EditorView, from: number, to: number, text: string, insert: () => Transaction) => boolean, readonly ((view: EditorView, from: number, to: number, text: string, insert: () => Transaction) => boolean)[]>;
    static focusChangeEffect: Facet<(state: EditorState, focusing: boolean) => StateEffect<any>, readonly ((state: EditorState, focusing: boolean) => StateEffect<any>)[]>;
    static perLineTextDirection: Facet<boolean, boolean>;
    static exceptionSink: Facet<(exception: any) => void, readonly ((exception: any) => void)[]>;
    static updateListener: Facet<(update: ViewUpdate) => void, readonly ((update: ViewUpdate) => void)[]>;
    static editable: Facet<boolean, boolean>;
    static mouseSelectionStyle: Facet<import("./input").MakeSelectionStyle, readonly import("./input").MakeSelectionStyle[]>;
    static dragMovesSelection: Facet<(event: MouseEvent) => boolean, readonly ((event: MouseEvent) => boolean)[]>;
    static clickAddsSelectionRange: Facet<(event: MouseEvent) => boolean, readonly ((event: MouseEvent) => boolean)[]>;
    static decorations: Facet<import("./decoration").DecorationSet | ((view: EditorView) => import("./decoration").DecorationSet), readonly (import("./decoration").DecorationSet | ((view: EditorView) => import("./decoration").DecorationSet))[]>;
    static atomicRanges: Facet<(view: EditorView) => import("@codemirror/state").RangeSet<any>, readonly ((view: EditorView) => import("@codemirror/state").RangeSet<any>)[]>;
    static bidiIsolatedRanges: Facet<import("./decoration").DecorationSet | ((view: EditorView) => import("./decoration").DecorationSet), readonly (import("./decoration").DecorationSet | ((view: EditorView) => import("./decoration").DecorationSet))[]>;
    static scrollMargins: Facet<(view: EditorView) => Partial<Rect>, readonly ((view: EditorView) => Partial<Rect>)[]>;
    static theme(spec: {
        [selector: string]: StyleSpec;
    }, options?: {
        dark?: boolean;
    }): Extension;
    static darkTheme: Facet<boolean, boolean>;
    static baseTheme(spec: {
        [selector: string]: StyleSpec;
    }): Extension;
    static cspNonce: Facet<string, string>;
    static contentAttributes: Facet<AttrSource, readonly AttrSource[]>;
    static editorAttributes: Facet<AttrSource, readonly AttrSource[]>;
    static lineWrapping: Extension;
    static announce: import("@codemirror/state").StateEffectType<string>;
    static findFromDOM(dom: HTMLElement): EditorView | null;
}
export interface DOMEventMap extends HTMLElementEventMap {
    [other: string]: any;
}
export declare type DOMEventHandlers<This> = {
    [event in keyof DOMEventMap]?: (this: This, event: DOMEventMap[event], view: EditorView) => boolean | void;
};
